package gonganxitong

import (
	"shylinux.com/x/ice"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/community/src/gonganxitong/model"
)

type allow struct {
	Table
	user    user
	event   event
	apply   apply
	order   string `data:"93"`
	role    string `data:"creator"`
	create  string `name:"create user_uid* apply_uid* status*"`
	reject  string `name:"reject" role:"void"`
	approve string `name:"approve" role:"void"`
}

func (s allow) Create(m *ice.Message, arg ...string) {
	s.Table.Create(m, arg...)
	s.SendMessage(m, m.Option(model.FROM_USER_UID), m.Option(model.USER_UID))
	s.sendTemplate(m, m.Trans("role order create", "权限审批 请处理"))
}
func (s allow) List(m *ice.Message, arg ...string) {
	s.Tables(m, s.apply, "left join users on applies.user_uid = users.uid").FieldsWithCreatedAT(m, s.apply,
		model.USER_AVATAR, model.USER_NAME,
		model.PLACE_UID, s.AS(s.Key(s.apply, model.ROLE), s.Keys(s.UserPlace, model.ROLE)), model.ALLOW_STATUS,
		model.BEGIN_TIME, model.END_TIME, s.AS(s.Key(s, model.UID), model.ALLOW_UID),
	)
	if len(arg) == 1 {
		s.Table.Select(m, s.Key(s, model.USER_UID), m.Option(model.USER_UID), model.PLACE_UID, arg[0])
		defer m.Sort(kit.Fields(model.ALLOW_STATUS, model.CREATED_AT), []string{AllowCreate.String(), AllowApproved.String(), AllowRejected.String()}, ice.STR_R)
	} else if len(arg) == 2 {
		s.Table.SelectDetail(m, s.Key(s, model.USER_UID), m.Option(model.USER_UID), model.PLACE_UID, arg[0], s.Key(s.apply, model.UID), arg[1])
	} else {
		return
	}
	s.RenameAppend(m, model.PLACE_UID, s.Keys(s.Place, model.UID)).SelectJoin(m, s.Place, model.NAME, model.TYPE)
	m.Table(func(value ice.Maps) {
		if AllowStatus(kit.Int(value[model.ALLOW_STATUS])) == AllowCreate {
			m.PushButton(s.Reject, s.Approve)
		} else {
			s.DoneMessage(m)
			m.PushButton()
		}
	}).Action()
	s.DisplayBase(m, "")
}
func (s allow) Reject(m *ice.Message, arg ...string) {
	s.process(m, AllowCreate, AllowRejected, m.Trans("role order rejected", "权限审批 已驳回"))
}
func (s allow) Approve(m *ice.Message, arg ...string) {
	s.process(m, AllowCreate, AllowApproved, m.Trans("role allow approved", "权限审批 已通过"))
}
func (s allow) process(m *ice.Message, from, to AllowStatus, title string) {
	defer m.ToastProcess()()
	if s.changeStatus(m, from, to); m.IsErr() {
		return
	}
	m.Cmdy(m.Prefix("apply"), m.ActionKey(), m.Option(model.UID)).ProcessRefresh()
	s.RecordEvent(m, "", kit.JoinWord("✅", title, kit.Cut(m.Option(model.UID), 6), s.TransRole(m), m.Option(model.USER_NAME)), m.Option(model.UID))
	s.DoneMessage(m)
	// s.sendTemplate(m, title)
}

func init() { ice.TeamCtxCmd(allow{Table: newTable()}) }

func (s allow) changeStatus(m *ice.Message, from, todo AllowStatus, arg ...string) *ice.Message {
	msg := s.Table.Select(m.Spawn(), model.UID, m.Option(model.ALLOW_UID), model.USER_UID, m.Option(model.USER_UID))
	if !m.WarnNotFound(msg.Length() == 0, m.Option(model.UID)) {
		if status := AllowStatus(kit.Int(msg.Append(model.STATUS))); !m.WarnNotValid(status != from, status.String()) {
			s.Table.Update(m, kit.Dict(model.STATUS, todo), model.UID, m.Option(model.ALLOW_UID))
		}
	}
	return msg
}
func (s allow) sendTemplate(m *ice.Message, info string) *ice.Message {
	s.Table.SendTemplate(m, m.Option(model.USER_NAME), m.Append(model.USER_UID), info)
	return m
}

type AllowStatus int

const (
	AllowCreate AllowStatus = iota
	AllowRejected
	AllowApproved
)

var AllowStatusList = map[AllowStatus]string{
	AllowCreate:   "create",
	AllowRejected: "rejected",
	AllowApproved: "approved",
}

func (s AllowStatus) String() string { return AllowStatusList[s] }
